/*
   SampleApp.cc.prg
   A sample dBL program from 
   The dBASE Book
   by Kenneth J. Mayer. 
   Program copyright 2005-2007. 
   Permission granted for use as a learning tool for dBASE Plus.

   Sample from: Chapter 24
   This is the application object used to start/close an application,
   trimmed down from work done on a recent project by myself.

   Dependencies: Setup.prg
                 MainMenu.mnu
                 :dUFLP:Ini.cc
*/
#define APP_VERSION	"1.0"
#define APP_NAME		" A Sample Application" // note the leading space

class SampleApp

   // Set up procedures in memory -- calling the SETUP.PRG file:
   if not setup()
   	release object this
      return
   endif

   // assign the instance of this class
   // as an object reference off the _app object:
   _app.Sample = this 


   /*------------------------------------------------
      This method handles opening the application
   ------------------------------------------------*/
   function open

      // set the menu:
      set procedure to MainMenu.mnu additive
      this.rootMenu = new MainMenuMenu( _app.framewin, "Root" )

      // You could insert code to load a toolbar here.

      // Set Shell
      shell( false, true )

      // Save titlebar text:
      this.OldTitleText = _app.Framewin.text

      // set the speedbar/statusbar off
      _app.Speedbar  := false // turn it off
      _app.StatusBar := false // same

      // if you save specific settings into a .INI file,
      // this might be a good place to read them, using
      // a file object, or a .ini class (such as the 
      // one in the dBASE Users' Function Library Project),
      // or code similar to what is used in the chapter
      // of the book dealing with Windows API calls.
      // use .INI file to check for a couple of user preferences:
      set procedure to :dUFLP:INI.cc additive
      _app.ini = new ini( set("directory")+"\SampleApp.ini" )
      // set speedtip preference:
      cSection = "Preferences"
      cEntry   = "ShowSpeedTips"
      // if it doesn't exist, create it with default:
      if empty( _app.ini.getValue( cSection, cEntry ) )
         _app.ini.setValue( cSection, cEntry, "true" )
      endif
      // get value, and set (convert string to logical) ...
      this.ShowSpeedTips = ;
           iif( _app.ini.getValue( cSection, cEntry ) == "true",;
                                   true, false )
      // minor note about the above code -- it was written
      // for an application that allows the user through the
      // use of a menu option and a dialog form, to set
      // this preference there. That is not included here.

      /* 
         disable the 'x' for _app ONLY if we're
         NOT in runtime -- only problem is that
         this leaves you in that state, and you
         have to "QUIT" dBASE ... BUT it does force
         a user not paying attention to use
         the menu to exit. It's not necessary for
         the compiled version.
      */
      if NOT "RUNTIME" $ upper( version(1) )
         sysClose( _app.Framewin, false ) // disable
      endif

      // display version and text in titlebar ....
      _app.FrameWin.text := APP_NAME + " -- " + APP_VERSION

      // make sure application is "on top"
      if type('BringWindowToTop') # 'FP'
      	extern clogical BringWindowToTop( cHandle ) user32
      endif
      BringWindowToTop( _app.framewin.hWnd )

   return

   /*------------------------------------------------
      Shutdown
   ------------------------------------------------*/
   function close

      // close anything we may have errored on and left
      // open:
      close forms

      // close any open data:
      close tables
      close databases

      // reset titlebar:
      _app.Framewin.text := this.OldTitleText
      this.OldTitleText := null

      // Save any settings to the application's .ini file
      // such as user preferences here.

      // release menu
      m = this.rootMenu
      release object m
      release m

      // if you loaded a toolbar in the open() method,
      // it would be prudent to detach it here, and release
      // the object reference.


      // release application object
      release object this
      _app.Sample := null

      // restore _app.framewin ...
      shell( true, true )

      // issue 'quit' only if we're in RUNTIME, otherwise
      // testing gets really annoying ...
      // If you don't issue the 'quit', then the shell
      // gets left on screen, which is weird, to say the least.
      if "RUNTIME" $ upper( version(1) )
         quit
      else
         // reset:
         _app.speedbar  := true
         _app.statusbar := true
         sysClose( _app.Framewin, true ) // enable close button
         keyboard('{ctrl+tab}')
      endif
   return

endclass

/* 
   ---------------------------------------------------------------
   Enables or disables the "close" item on the system menu as well 
   as the Win95 X.

      Syntax:    sysClose( oForm, lEnabled )
      Example:   sysClose( _app.FrameWin, false ) // disable
                 sysClose( _app.FrameWin, true )  // ensable

   Posted by Gary White in the VdBASE newsgroups
   7/3/1998

   Returns the previous state of the menu item.
   ---------------------------------------------------------------
*/
function sysClose
parameters oForm, lEnabled
local hMenu, lRetVal
#define MF_ENABLED      0x00000000
#define MF_GRAYED       0x00000001
#define SC_CLOSE        0xF060

   if argCount() # 2 or ;
         type("oForm.hWnd") # "N" ;
         or type("lEnabled") # "L"
      msgbox( "Correct syntax is:  sysClose( oForm, lEnabled )", ;
         "Error", 16 )
      return
   endif
   if empty( oForm.hWnd )
      msgbox( "Form is not open", "Error", 16 )
      return
   endif

   if type( "GetSystemMenu" ) # "FP"
     extern cint GetSystemMenu( cHandle, cInt ) USER32
   endif
   if type( "EnableMenuItem" ) # "FP"
      extern CLOGICAL EnableMenuItem( cHandle, cInt, cInt) user32
   endif
	hMenu = GetSystemMenu( oForm.hWnd, 0 )
   if lEnabled
   	lRetVal = EnableMenuItem( hMenu, SC_CLOSE, MF_ENABLED )
   else
   	lRetVal = EnableMenuItem( hMenu, SC_CLOSE, MF_GRAYED )
	endif
return not lRetVal

